@@ -1,5 +1,5 @@
 #
-# Copyright 2012-2020 Red Hat, Inc.
+# Copyright 2012-2021 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -470,6 +470,11 @@ class ResetFailed(ContextException):
     message = "Failed to reset VM."
 
 
+class InvalidParameter(ContextException):
+    code = 91
+    message = 'Invalid parameter'
+
+
 class RecoveryInProgress(VdsmException):
     code = 99
     message = 'Recovering from crash or Initializing'
